package com.dayou.controller;

import com.dayou.annotation.IgnoreAuth;
import com.dayou.common.RestResponse;
import com.dayou.configuration.DfsConfig;
import com.dayou.exception.ErrorCode;
import com.dayou.exception.file.InvalidExtensionException;
import com.dayou.service.IGlobalConfigService;
import com.dayou.utils.FileUploadUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import java.util.HashMap;
import java.util.Map;
import java.util.regex.Pattern;

import static com.dayou.exception.ErrorCode.DEFAULTERROR;
import static com.dayou.exception.ErrorCode.FILE_EXT_ERROR;


/**
 * 文件上传
 *
 * @author wucl
 * @since 2023-2-7
 */
@Slf4j
@RestController
@IgnoreAuth
public class FileController {

    private static final Pattern PORTPATTERN = Pattern.compile(":\\d+");

    @Autowired
    private DfsConfig dfsConfig;


    @Value("${server.port:80}")
    private Integer port;

    /**
     * 通用上传请求
     */
    @PostMapping("upload")
    public RestResponse upload(MultipartFile file) {
        try {
            // 上传文件路径
            String filePath = dfsConfig.getPath();
            // 上传并返回新文件名称
            String fileName = null;
            fileName = FileUploadUtils.upload(filePath, file);
            String url = dfsConfig.getDomain() + fileName;
            Map map = new HashMap();
            map.put("fileName", fileName);
            map.put("url", url);
            return RestResponse.data(map);
        } catch (InvalidExtensionException e) {
            ErrorCode.throwBusinessException(FILE_EXT_ERROR);
        } catch (Exception e) {
            log.error("upload error", e);
            ErrorCode.throwBusinessException(DEFAULTERROR);
        }

        return RestResponse.error();
    }

}