package com.dayou.exception;

import org.apache.commons.lang3.ArrayUtils;
import org.apache.commons.lang3.builder.ToStringBuilder;

import java.util.Objects;

/**
 * Define the error id.
 */
public class ErrorCode {
    public static final ErrorCode PWD_ERROR = ErrorCode("001", "账号/密码错误");
    public static final ErrorCode FORM_EXPIRED = ErrorCode("002", "页面过期，请刷新后重试");
    public static final ErrorCode FORM_DUPLICATE = ErrorCode("003", "请勿重复提交");
    public static final ErrorCode DATA_NOT_EXISTS = ErrorCode("004", "数据不存在");

    public static final ErrorCode RAS_IMPORT_ERROR = ErrorCode("10000", "导入excel出错 {0}");

    public static final ErrorCode NO_AUTH = ErrorCode("10001", "用户无权限");
    public static final ErrorCode UN_LOGIN = ErrorCode("10004", "用户未登录");
    public static final ErrorCode LOGIN_EXPIRE = ErrorCode("10005", "登录过期");
    public static final ErrorCode DEFAULTERROR = ErrorCode("10006", "系统繁忙");
    public static final ErrorCode PARAM_ERROR = ErrorCode("10007", "参数错误");

    public static final ErrorCode INVALID_TOKEN = ErrorCode("10008", "非法token");
    public static final ErrorCode DATA_EXIST = ErrorCode("10009", "数据已存在");

    public static final ErrorCode CUSTOM_ERROR = ErrorCode("10010", "{0}");

    public static final ErrorCode USER_DISABLE = ErrorCode("10011", "用户被禁用,请联系管理员");
    public static final ErrorCode STAGE_NODE_ERROR = ErrorCode("10012", "请选择上个阶段");
    public static final ErrorCode STAGE_NODE_ERROR1 = ErrorCode("10013", "不能选择自己为上阶段");

    public static final ErrorCode PASSWORD_DIFFERENT = ErrorCode("10014", "两次输入的密码不一致,请检查.");

    public static final ErrorCode PASSWORD_ERROR= ErrorCode("10015", "原密码错误,请检查.");

    public static final ErrorCode FILE_EXT_ERROR = ErrorCode("10016", "不支持此文件扩展名");

    public static final ErrorCode BROKERAGE_ITEM_STATE_ERROR = ErrorCode("10017", "项目已被结算或冻结，无法计算提成");

    public static final ErrorCode BROKERAGE_ITEM_STATE_ERROR1 = ErrorCode("10018", "项目未开始，无法计算提成");

    public static final ErrorCode BROKERAGE_ITEM_STATE_ERROR2 = ErrorCode("10019", "客户经理未达到回款目标，无法计算提成");

    public static final ErrorCode BROKERAGE_SETTLE_FAILED = ErrorCode("10020", "项目结算失败，请检查项目排期或提成权重。");

    public static final ErrorCode ITEM_EVALUATE_ERROR = ErrorCode("10021", "当前项目仍有未确认是否逾期的阶段，请先确认后再评价。");




    private String code;
    private String errorMsg;

    public static ErrorCode ErrorCode(String code, String errorMsg) {
        return new ErrorCode(code, errorMsg);
    }

    private ErrorCode() {
    }


    public ErrorCode(String code, String errorMsg) {
        this.code = code;
        this.errorMsg = errorMsg;
    }

    public static void throwBusinessException(ErrorCode errorCode, Object... values) {
        throw buildBusinessException(errorCode, values);
    }

    public static BusinessException buildBusinessException(ErrorCode errorCode, Object... values) {
        String message = errorCode.errorMsg;
        if (ArrayUtils.isNotEmpty(values)) {
            for (int i = 0; i < values.length; i++) {
                if (message.contains("{") && message.contains("}")) {
                    message = message.replaceFirst("\\{" + i + "\\}", values[i].toString());
                } else {
                    message += " " + values[i].toString() + " ";
                }
            }
        }

        return new BusinessException(message, errorCode.code);
    }

    public static void throwBusinessException(String msg, String code) {
        throw new BusinessException(msg, code == null ? DEFAULTERROR.getCode() : code);
    }


    public String getCode() {
        return code;
    }

    public String getErrorMsg() {
        return errorMsg;
    }

    @Override
    public String toString() {
        return new ToStringBuilder(this)
                .append("id", code)
                .append("errorMsg", errorMsg)
                .toString();
    }

    public void setCode(String code) {
        this.code = code;
    }

    public void setErrorMsg(String errorMsg) {
        this.errorMsg = errorMsg;
    }

    public static void throwExceptionIfNull(Object o){
        if(Objects.isNull(o)){
            throwBusinessException(DATA_NOT_EXISTS);
        }
    }
}
