package com.dayou.annotation;

import java.lang.annotation.*;

/**
 * 增强排序
 * descs=weekScore   weekScore字段可能在数据库或你写的sql 中不存在这个字段 就会报错
 * 实际上mybatis 把week_score 映射成了 weekScore
 * 此注解作用就是自动做一个转换
 * 驼峰-> 下划线
 * 下划线-> 驼峰
 */
@Target({ElementType.METHOD})
@Retention(RetentionPolicy.RUNTIME)
@Documented
public @interface EnchanceSort {

    /**
     * 需要排除的字段 多个逗号连接即可;
     * 默认对传来的所有排序字段转化成 underline格式 例如
     * userId -> user_id
     * age-> age
     * Age->age
     * weekScore-> week_score
     * big_id -> big_id
     *
     * @return
     */
    String excluds() default "";

    /**
     * 默认转化为下划线模式
     * 否则转为驼峰模式
     *
     * @return
     */
    boolean underlineMode() default true;


}
