package com.dayou.aspect;

import com.dayou.annotation.DataPermission;
import com.dayou.bo.SimpleParentModel;
import com.dayou.common.SimpleListModel;
import com.dayou.dto.SimplePostModel;
import com.dayou.service.IPostService;
import com.dayou.service.IUserService;
import com.dayou.utils.LoginContext;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.lang.reflect.Field;
import java.util.*;
import java.util.stream.Collectors;

/**
 * 类说明：
 *
 * @author: wucl
 * @since: 2023/2/13
 * created with IntelliJ IDEA.
 */

@Slf4j
@Component
@Aspect
public class DataPermissionAop {

    @Autowired
    private IUserService userService;

    @Autowired
    private IPostService postService;

    @Pointcut("@annotation(com.dayou.annotation.DataPermission)")
    public void dataPermissionPointcut(){

    }

    @Around("dataPermissionPointcut() && @annotation(dataPermission)")
    public Object  beforeHandle(ProceedingJoinPoint joinPoint, DataPermission dataPermission) throws Throwable {

        Object arg = joinPoint.getArgs()[0];

        if (dataPermission.department()){
            List<SimpleParentModel> departmentList = LoginContext.getLoginCacheUserBO().getDepartmentList();
            List<Long> ids = departmentList.stream().map(SimpleListModel::getId).collect(Collectors.toList());
            List<Long> parentIds = departmentList.stream().map(SimpleParentModel::getParentId).collect(Collectors.toList());
            //需特殊处理最高部门,最高部门无数据权限限制
            if (parentIds.contains(null)){
                return joinPoint.proceed();
            }
            //部门内部最高岗位拥有该部门最高数据权限，否则只有登录人参与的项目的数据权限
            List<SimplePostModel> postList = LoginContext.getLoginCacheUserBO().getPostList();

            List<Long> dIds = new ArrayList<>();
            Set<Long> users = new HashSet<>();
            for (SimplePostModel post : postList){
                if (postService.mostHighPost(post)){
                    dIds.add(post.getDepartmentId());
                }else {
                    users.add(LoginContext.getUserId());
                }
            }

            Field departments = arg.getClass().getDeclaredField("departmentIds");
            departments.setAccessible(Boolean.TRUE);
            departments.set(arg,dIds);
            Field userIds = arg.getClass().getDeclaredField("userIds");
            userIds.setAccessible(Boolean.TRUE);
            userIds.set(arg,users);
        }

        if (dataPermission.postChild()){
            Set<Long> users = new HashSet<>();
            Long userId = LoginContext.getUserId();
            //根据当前登录人获取下属员工id
            users = userService.getJuniorUserId(userId);
            users.add(userId);
            Field userIds = arg.getClass().getDeclaredField("userIds");
            userIds.setAccessible(Boolean.TRUE);
            userIds.set(arg,users);
        }

        return joinPoint.proceed();

    }
}
