package com.dayou.service.impl;

import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.dayou.bo.BrokerageDetailBO;
import com.dayou.bo.LeaderRatioBO;
import com.dayou.brokerage.BrokerageCalculateSupport;
import com.dayou.brokerage.MarketerBrokerageCalculator;
import com.dayou.brokerage.NonMarketerMarketerBrokerageCalculator;
import com.dayou.brokerage.constants.BrokerageMode;
import com.dayou.brokerage.constants.BrokerageRule;
import com.dayou.brokerage.constants.BrokerageState;
import com.dayou.common.BaseEntity;
import com.dayou.dto.ItemDTO;
import com.dayou.dto.ParticipantSettleDTO;
import com.dayou.entity.*;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.*;
import com.dayou.service.*;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.vo.ItemStageVO;
import com.google.common.collect.Sets;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;

import java.math.BigDecimal;
import java.util.HashSet;
import java.util.List;
import java.util.Set;
import java.util.stream.Collectors;

import org.springframework.transaction.annotation.Transactional;

/**
 * <p>
 * 项目提成总表 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2023-03-03
 */
@Slf4j
@Service
public class ItemBrokerageGeneralServiceImpl extends ServiceImpl<ItemBrokerageGeneralMapper, ItemBrokerageGeneral> implements IItemBrokerageGeneralService {


    @Autowired
    private BrokerageCalculateSupport brokerageCalculateSupport;

    @Autowired
    private IUserTargetService userTargetService;

    @Autowired
    private IItemBrokerageDetailService iItemBrokerageDetailService;

    @Autowired
    private IMarketStatService marketStatService;

    @Autowired
    private IUserService userService;

    @Autowired
    private ItemMapper itemMapper;

    @Autowired
    private ItemStageMapper itemStageMapper;

    @Autowired
    private ItemEvaluateMapper itemEvaluateMapper;

    @Autowired
    private PaymentCollectionMapper paymentCollectionMapper;

    @Override
    @SuppressWarnings("unchecked")
    public Page<ItemBrokerageGeneral> selectPage(Page page,ItemBrokerageGeneral itemBrokerageGeneral){
        return this.page(page, new QueryWrapper<ItemBrokerageGeneral>(itemBrokerageGeneral));
    }


    @Override
    public ItemBrokerageGeneral detail(Long id){
        return this.getById(id);
    }

    @Override
    public Boolean add(ItemBrokerageGeneral itemBrokerageGeneral){
        return  this.save(itemBrokerageGeneral);
    }

    @Override
    public Boolean update(ItemBrokerageGeneral itemBrokerageGeneral){
        return  this.updateById(itemBrokerageGeneral);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Transactional
    @Override
    public void doCreateBrokerageGeneralAndDetail(BrokerageDetailBO bo) {
        MarketerBrokerageCalculator calculator = (MarketerBrokerageCalculator) brokerageCalculateSupport.getCalculator(BrokerageRule.LAND_MARKETER_RULE);
            Long userId = bo.getUserId();
        UserTarget userTarget = getUserTarget(userId);
        BigDecimal currentMonthPayment = marketStatService.currentYearPayment(userId);
        BigDecimal paymentTarget = new BigDecimal(userTarget.getPaymentTarget());
        bo.setCurrentPayment(currentMonthPayment);
        bo.setPaymentTarget(paymentTarget);
        Set<LeaderRatioBO> leaderRatioBO = userService.getLeaderRatioBO(userId).stream().filter(x->x.getRatio()!=null).collect(Collectors.toSet());
        bo = calculator.predictPersonalAmount(bo);
        //创建下单人领导提成记录
        Set<ItemBrokerageDetail> brokerageLeaderDetailBOS = calculator.predictTeamShareAmount(bo, leaderRatioBO);
        BigDecimal leaderPredictTotalAmount = brokerageLeaderDetailBOS.stream().map(ItemBrokerageDetail::getPredictAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
        //构建提成总表对象
        ItemBrokerageGeneral general = ItemBrokerageGeneral.builder()
                .itemId(bo.getItemId())
                .marketerBrokerageAmount(bo.getPredictAmount().add(leaderPredictTotalAmount))
                .marketerAdvanceBrokerageAble(bo.getMarketerAdvanceBrokerageAble())
                .participantActualBrokerageAmount(BigDecimal.ZERO)
                .participantPredictBrokerageAmount(BigDecimal.ZERO)
                .brokeragePredictAmount(bo.getPredictAmount().add(leaderPredictTotalAmount))
                .brokerageActualAmount(BigDecimal.ZERO)
                .participantStatus(BrokerageState.NOT_START.getCode())
                .marketerStatus(bo.getMarketerStatus())
                .brokerageRate(bo.getRate()).build();
        this.save(general);
        brokerageLeaderDetailBOS.add(bo);
        brokerageLeaderDetailBOS.stream().forEach(x->{ x.setGeneralId(general.getId());});
        iItemBrokerageDetailService.saveBatch(brokerageLeaderDetailBOS);
    }

    @Override
    public void addMarketerBrokerage(Long id) {
        Item item = itemMapper.selectById(id);
        if (item.getAmount()!=null){
            BrokerageDetailBO brokerageDetailBO = BrokerageDetailBO.builder().amount(item.getAmount()).cate(item.getCate()).businessSource(item.getBusinessSource()).itemId(id).build();
            brokerageDetailBO.setBrokerageRule(BrokerageRule.LAND_MARKETER_RULE.getCode());
            brokerageDetailBO.setAdvanceAmount(BigDecimal.ZERO);
            brokerageDetailBO.setPredictAmount(BigDecimal.ZERO);
            brokerageDetailBO.setActualAmount(BigDecimal.ZERO);
            brokerageDetailBO.setAheadAmount(BigDecimal.ZERO);
            brokerageDetailBO.setMarketerStatus(BrokerageState.NOT_PAYMENT.getCode());
            brokerageDetailBO.setUserId(item.getUserId());
            this.doCreateBrokerageGeneralAndDetail(brokerageDetailBO);
        }else {
            log.info("未设置合同金额，创建提成总表和客户经理提成记录终止。itemId[{}]",id);
        }

    }

    @Transactional
    @Override
    public void editMarketerBrokerage(ItemDTO itemDTO) {
        ItemBrokerageGeneral general = this.getOne(new LambdaQueryWrapper<ItemBrokerageGeneral>().eq(ItemBrokerageGeneral::getItemId, itemDTO.getId()).eq(BaseEntity::getDeleted,Boolean.FALSE));
        ItemBrokerageDetail marketerDetail = iItemBrokerageDetailService.getOne(new LambdaQueryWrapper<ItemBrokerageDetail>().eq(ItemBrokerageDetail::getGeneralId, general.getId()).eq(BaseEntity::getDeleted, Boolean.FALSE).eq(ItemBrokerageDetail::getBrokerageMode,BrokerageMode.PERSONAL.getCode()));
        if (general == null || marketerDetail == null){
            addMarketerBrokerage(itemDTO.getId());
            log.info("更新项目信息时创建了相关的提成记录");
            return;
        }

        MarketerBrokerageCalculator calculator = (MarketerBrokerageCalculator) brokerageCalculateSupport.getCalculator(BrokerageRule.LAND_MARKETER_RULE);
        //更新客户经理的提成记录
        UserTarget userTarget = getUserTarget(marketerDetail.getUserId());
        BigDecimal currentMonthPayment = marketStatService.currentMonthPayment(marketerDetail.getUserId());
        BigDecimal paymentTarget = new BigDecimal(userTarget.getPaymentTarget());
        BrokerageDetailBO brokerageDetailBO = BrokerageDetailBO.builder().amount(itemDTO.getAmount()).cate(itemDTO.getCate()).businessSource(itemDTO.getBusinessSource()).itemId(itemDTO.getId()).paymentTarget(paymentTarget).currentPayment(currentMonthPayment).build();
        brokerageDetailBO.setUserId(marketerDetail.getUserId());
        brokerageDetailBO.setId(marketerDetail.getId());
        brokerageDetailBO.setGeneralId(general.getId());
        brokerageDetailBO = calculator.predictPersonalAmount(brokerageDetailBO);
        iItemBrokerageDetailService.update(brokerageDetailBO);


        //更新领导提成记录
        Set<LeaderRatioBO> leaderRatioBO = userService.getLeaderRatioBO(marketerDetail.getUserId()).stream().filter(x->x.getRatio()!=null).collect(Collectors.toSet());
        Set<ItemBrokerageDetail> itemBrokerageDetails = calculator.predictTeamShareAmount(brokerageDetailBO, leaderRatioBO);
        iItemBrokerageDetailService.remove(new LambdaQueryWrapper<ItemBrokerageDetail>().eq(ItemBrokerageDetail::getGeneralId,general.getId()).eq(ItemBrokerageDetail::getBrokerageMode,BrokerageMode.TEAM_SHARE.getCode()));
        iItemBrokerageDetailService.saveBatch(itemBrokerageDetails);

        //更新总表
        BigDecimal leaderPredictTotalAmount = itemBrokerageDetails.stream().map(ItemBrokerageDetail::getPredictAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
        general.setMarketerBrokerageAmount(brokerageDetailBO.getPredictAmount().add(leaderPredictTotalAmount));
        general.setMarketerAdvanceBrokerageAble(brokerageDetailBO.getMarketerAdvanceBrokerageAble());
        general.setBrokerageRate(brokerageDetailBO.getRate());
        general.setBrokeragePredictAmount(brokerageDetailBO.getPredictAmount().add(general.getParticipantPredictBrokerageAmount()).add(leaderPredictTotalAmount));
        this.update(general);
    }

    @Transactional
    @Override
    public void removeMarketerBrokerage(Long itemId) {
        ItemBrokerageGeneral general = this.getOne(new LambdaQueryWrapper<ItemBrokerageGeneral>().eq(ItemBrokerageGeneral::getItemId, itemId));
        this.removeById(general.getId());
        iItemBrokerageDetailService.remove(new LambdaQueryWrapper<ItemBrokerageDetail>().eq(ItemBrokerageDetail::getGeneralId,general.getId()));
    }

    @Transactional
    @Override
    public void addParticipatorBrokerage(ItemStageVO itemStageVO,Long brokerageStageId) {
        Item item = itemMapper.selectById(itemStageVO.getItemId());
        if (item.getAmount()!=null){
            NonMarketerMarketerBrokerageCalculator calculator = (NonMarketerMarketerBrokerageCalculator) brokerageCalculateSupport.getCalculator(BrokerageRule.LAND_OTHER_RULE);
            ItemBrokerageGeneral general = this.getOne(new LambdaQueryWrapper<ItemBrokerageGeneral>().eq(ItemBrokerageGeneral::getItemId, itemStageVO.getItemId()));
            if (general == null){
                addMarketerBrokerage(itemStageVO.getItemId());
                addParticipatorBrokerage(itemStageVO,brokerageStageId);
                log.info("更新项目信息时创建了相关的提成记录");
                return;
            }
            Set<ItemStageVO.ParticipatorWeight> participators = itemStageVO.getParticipators();
            Set<ItemBrokerageDetail> participatorBrokerageDetails = participators.stream().map(x -> {
                BrokerageDetailBO brokerageDetailBO = BrokerageDetailBO.builder().build();
                brokerageDetailBO.setBrokerageRule(BrokerageRule.LAND_OTHER_RULE.getCode());
                brokerageDetailBO.setBrokerageMode(BrokerageMode.PERSONAL.getCode());
                brokerageDetailBO.setAdvanceAmount(BigDecimal.ZERO);
                brokerageDetailBO.setPredictAmount(BigDecimal.ZERO);
                brokerageDetailBO.setActualAmount(BigDecimal.ZERO);
                brokerageDetailBO.setAmount(item.getAmount());
                brokerageDetailBO.setUserId(x.getUserId());
                brokerageDetailBO.setGeneralId(general.getId());
                brokerageDetailBO.setBrokerageStageId(brokerageStageId);
                brokerageDetailBO.setStageWeight(itemStageVO.getWeight());
                brokerageDetailBO.setParticipantWeight(x.getWeight());
                ItemBrokerageDetail itemBrokerageDetail = calculator.predictAmount(brokerageDetailBO);
                return itemBrokerageDetail;
            }).collect(Collectors.toSet());
            BigDecimal participatorPredictTotalAmount = participatorBrokerageDetails.stream().map(ItemBrokerageDetail::getPredictAmount).reduce(BigDecimal.ZERO, BigDecimal::add);

            //创建参与人的项目提成详情
            iItemBrokerageDetailService.saveBatch(participatorBrokerageDetails);
            //更新项目提成总表信息
            general.setParticipantPredictBrokerageAmount(participatorPredictTotalAmount.add(general.getParticipantPredictBrokerageAmount()));
            general.setParticipantAdvanceStatus(Boolean.TRUE);
            general.setBrokeragePredictAmount(general.getMarketerBrokerageAmount().add(general.getParticipantPredictBrokerageAmount()));
            general.setParticipantActualBrokerageAmount(BigDecimal.ZERO);
            general.setParticipantStatus(BrokerageState.PREDICTING.getCode());

            this.updateById(general);

        }else {
            log.info("未设置合同金额，创建参与人提成记录终止。itemId[{}]",item.getId());
        }


    }

    @Override
    public void editParticipatorBrokerage(ItemStageVO itemStageVO) {
        iItemBrokerageDetailService.remove(new LambdaQueryWrapper<ItemBrokerageDetail>().eq(ItemBrokerageDetail::getBrokerageStageId,itemStageVO.getId()));
        addParticipatorBrokerage(itemStageVO,itemStageVO.getId());
    }

    @Transactional
    @Override
    public void removeParticipatorBrokerage(Long itemStageId) {
        List<ItemBrokerageDetail> list = iItemBrokerageDetailService.list(new LambdaQueryWrapper<ItemBrokerageDetail>()
                .eq(ItemBrokerageDetail::getBrokerageStageId, itemStageId));
        if (CollectionUtils.isEmpty(list)){
            return ;
        }
        Long generalId = list.get(0).getGeneralId();
        if (generalId == null){
            return ;
        }
        ItemBrokerageGeneral general = this.getById(generalId);
        BigDecimal participatorPredictTotalAmount = list.stream().map(ItemBrokerageDetail::getPredictAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
        general.setParticipantPredictBrokerageAmount(general.getParticipantPredictBrokerageAmount().subtract(participatorPredictTotalAmount));
        general.setBrokeragePredictAmount(general.getBrokeragePredictAmount().subtract(participatorPredictTotalAmount));
       this.updateById(general);
        iItemBrokerageDetailService.remove(new LambdaQueryWrapper<ItemBrokerageDetail>().eq(ItemBrokerageDetail::getBrokerageStageId,itemStageId));
    }

    @Transactional
    @Override
    public void settleParticipatorBrokerage(Long itemId) {
        ItemBrokerageGeneral general = this.getOne(new LambdaQueryWrapper<ItemBrokerageGeneral>().eq(ItemBrokerageGeneral::getItemId, itemId).eq(BaseEntity::getDeleted, Boolean.FALSE));
        List<ItemStage> stages = itemStageMapper.selectList(new LambdaQueryWrapper<ItemStage>().eq(ItemStage::getItemId, itemId));
        ItemEvaluate settle = itemEvaluateMapper.selectOne(new LambdaQueryWrapper<ItemEvaluate>().eq(ItemEvaluate::getItemId, itemId).eq(BaseEntity::getDeleted, Boolean.FALSE));
        if (general!=null && CollectionUtils.isNotEmpty(stages) && settle!=null && settle.getWeight()!=null){
            NonMarketerMarketerBrokerageCalculator calculator = (NonMarketerMarketerBrokerageCalculator) brokerageCalculateSupport.getCalculator(BrokerageRule.LAND_OTHER_RULE);
            ParticipantSettleDTO participantSettle = ParticipantSettleDTO.builder().generalId(general.getId()).stages(stages).brokerageState(general.getParticipantStatus()).weight(settle.getWeight()).build();
            List<ItemBrokerageDetail> itemBrokerageDetails = calculator.actualAmount(participantSettle);
            iItemBrokerageDetailService.updateBatchById(itemBrokerageDetails);
            //结算后更新总表参与人提成总金额
            BigDecimal participatorActualTotalAmount = itemBrokerageDetails.stream().map(ItemBrokerageDetail::getActualAmount).reduce(BigDecimal.ZERO, BigDecimal::add);
            general.setParticipantActualBrokerageAmount(participatorActualTotalAmount);
            general.setParticipantAdvanceStatus(Boolean.FALSE);
            general.setBrokerageActualAmount(participatorActualTotalAmount.add(general.getBrokerageActualAmount()));
            general.setParticipantStatus(BrokerageState.CLOSED.getCode());
            this.updateById(general);
        }else {
            log.info(ErrorCode.BROKERAGE_SETTLE_FAILED.getErrorMsg()+"项目id:[{}]",itemId);
        }

    }

    @Override
    public void settleMarketerBrokerage(PaymentCollection payment) {
        ItemBrokerageGeneral general = this.getOne(new LambdaQueryWrapper<ItemBrokerageGeneral>().eq(ItemBrokerageGeneral::getItemId, payment.getItemId()).eq(BaseEntity::getDeleted, Boolean.FALSE));
        Item item = itemMapper.selectById(payment.getItemId());
        List<PaymentCollection> paymentList = paymentCollectionMapper.selectList(new LambdaQueryWrapper<PaymentCollection>()
                .eq(PaymentCollection::getItemId, payment.getItemId()).eq(BaseEntity::getDeleted, Boolean.FALSE));
        BigDecimal payDone = BigDecimal.valueOf(paymentList.stream().mapToDouble(PaymentCollection::getAmount).sum());
        brokerageCalculateSupport.checkItemBrokerageStatusOnMarketerSettle(general.getMarketerStatus());
        if (item.getAmount()!=null && item.getAmount().compareTo(payDone)==0){
            general.setMarketerStatus(BrokerageState.NOT_SETTLE.getCode());
            this.updateById(general);
        }
        log.info("还有未回款项，暂不执行客户经理提成结算。");
    }

    /**
     * 获取计算提成的项目 (待预提，待结算)
     * @return
     */
    @Override
    public List<ItemBrokerageGeneral> computedAbleList() {
        HashSet<String> status = Sets.newHashSet();
        status.add(BrokerageState.TO_PREDICTING.getCode());
        status.add(BrokerageState.NOT_SETTLE.getCode());
        List<ItemBrokerageGeneral> ableSettleList = this.list(new LambdaQueryWrapper<ItemBrokerageGeneral>().in(ItemBrokerageGeneral::getMarketerStatus,status)
                .or().eq(ItemBrokerageGeneral::getParticipantStatus,BrokerageState.NOT_SETTLE.getCode()));
        return ableSettleList;
    }

    /**
     * 获取客户经理年回款目标
     * @param userId
     * @return
     */
    private UserTarget getUserTarget(Long userId) {
        UserTarget userTarget = userTargetService.getOne(new LambdaQueryWrapper<UserTarget>().eq(UserTarget::getUserId, userId));
        if (userTarget==null || userTarget.getPaymentTarget()==null){
            userTarget = new UserTarget();
            userTarget.setPaymentTarget(0d);
            log.info("客户经理id[{}]未设置年回款目标。", userId);
        }
        return userTarget;
    }
}
