package com.dayou.service.impl;

import com.dayou.brokerage.annotation.DoBrokerage;
import com.dayou.brokerage.constants.BrokerageBusiness;
import com.dayou.brokerage.constants.BrokerageRule;
import com.dayou.common.BaseEntity;
import com.dayou.entity.PaymentCollection;
import com.dayou.enums.OperationTypeEnum;
import com.dayou.exception.ErrorCode;
import com.dayou.mapper.PaymentCollectionMapper;
import com.dayou.service.IItemService;
import com.dayou.service.IPaymentCollectionService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.dayou.vo.ItemPaymentVO;
import com.dayou.vo.PaymentCollectionVO;
import com.sun.org.apache.xpath.internal.operations.Bool;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;

import java.math.BigDecimal;
import java.util.List;
import java.util.ArrayList;
import java.util.stream.Collectors;

import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

/**
 * <p>
 * 回款记录 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2023-01-10
 */
@Service
public class PaymentCollectionServiceImpl extends ServiceImpl<PaymentCollectionMapper, PaymentCollection> implements IPaymentCollectionService {

    @Autowired
    private PaymentCollectionMapper paymentCollectionMapper;

    @Autowired
    private IItemService iItemService;
    
    @Override
    @SuppressWarnings("unchecked")
    public Page<PaymentCollectionVO> selectPage(Page page, PaymentCollectionVO paymentCollection){
        return paymentCollectionMapper.page(page, paymentCollection);
    }


    @Override
    public PaymentCollection detail(Long id){
        return this.getById(id);
    }

    @DoBrokerage(operation = OperationTypeEnum.EDIT,rule = BrokerageRule.LAND_MARKETER_RULE,business = BrokerageBusiness.SETTLE)
    @Override
    public Boolean add(PaymentCollection paymentCollection){
        BigDecimal totalAmount = iItemService.getById(paymentCollection.getItemId()).getAmount();
        List<Double> doneAmount = this.list(new LambdaQueryWrapper<PaymentCollection>()
                .select(PaymentCollection::getAmount)
                .eq(PaymentCollection::getItemId, paymentCollection.getItemId())
                .eq(BaseEntity::getDeleted, Boolean.FALSE)).stream().map(PaymentCollection::getAmount).collect(Collectors.toList());


        BigDecimal sum = BigDecimal.valueOf(doneAmount.stream().reduce(0D, Double::sum));
        BigDecimal payment = BigDecimal.valueOf(paymentCollection.getAmount());
        BigDecimal add = sum.add(payment);
        if (add.compareTo(totalAmount)==1){
            ErrorCode.throwBusinessException(ErrorCode.CUSTOM_ERROR,"新增回款金额已大于合同签订金额，请检查.");
        }
        return  this.save(paymentCollection);
    }

    @Override
    public Boolean update(PaymentCollection paymentCollection){
        return  this.updateById(paymentCollection);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Override
    public IPage<ItemPaymentVO> getItemPayment(Page page,PaymentCollectionVO paymentCollectionVO) {
        IPage<ItemPaymentVO> result = paymentCollectionMapper.getItemPayment(page,paymentCollectionVO);
        return result;
    }
}
