package com.dayou.service.impl;

import com.dayou.common.BaseEntity;
import com.dayou.common.SimpleListModel;
import com.dayou.dto.SimplePostModel;
import com.dayou.entity.UserPost;
import com.dayou.mapper.UserPostMapper;
import com.dayou.service.IUserPostService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.dayou.utils.ExcelUtil;
import org.apache.poi.ss.usermodel.Cell;
import org.apache.poi.ss.usermodel.CellStyle;
import org.apache.poi.ss.usermodel.Row;
import org.apache.poi.ss.usermodel.Sheet;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.web.multipart.MultipartFile;
import java.util.List;
import java.util.ArrayList;
import org.springframework.transaction.annotation.Transactional;
import com.dayou.enums.BatchTaskTypeEnum;

/**
 * <p>
 * 用户岗位 服务实现类
 * </p>
 *
 * @author wucl
 * @since 2022-11-29
 */
@Service
public class UserPostServiceImpl extends ServiceImpl<UserPostMapper, UserPost> implements IUserPostService {

    private static final List<String> userPostExcelTitles= new ArrayList();

    @Autowired
    private UserPostMapper userPostMapper;


    @Override
    @SuppressWarnings("unchecked")
    public Page<UserPost> selectPage(Page page,UserPost userPost){
        return this.page(page, new QueryWrapper<UserPost>(userPost));
    }


    @Override
    public UserPost detail(Long id){
        return this.getById(id);
    }

    @Override
    public Boolean add(UserPost userPost){
        return  this.save(userPost);
    }

    @Override
    public Boolean update(UserPost userPost){
        return  this.updateById(userPost);
    }

    @Override
    public Boolean delete(Long id){
        //逻辑删除
        return this.removeById(id);
    }

    @Override
    public XSSFWorkbook generateImportTemplate(){
        XSSFWorkbook wb = new XSSFWorkbook();
        //创建固定列的表头
        createExcelTitle(wb);
        return wb;
    }

    private void createExcelTitle(XSSFWorkbook wb) {
        Sheet sheet = wb.createSheet("用户岗位导入模板");
        Row row = sheet.createRow(0);
        CellStyle style = wb.createCellStyle();
        Cell cell = null;
        for (int i = 0; i < userPostExcelTitles.size(); i++) {
            cell = row.createCell(i);
            cell.setCellStyle(style);
            cell.setCellValue(userPostExcelTitles.get(i));
        }
    }

    @Transactional(rollbackFor = Exception.class)
    @Override
    public void importExcel(MultipartFile file){
        ExcelUtil.importExcel(UserPost.class, new ExcelUtil.ImportListener<UserPost>() {
            @Override
            public Boolean handleOneRecord(UserPost userPost) {
                return add(userPost);
            }

            @Override
            public void afterImport(ExcelUtil.BatchImportResult batchImportResult) {

            }
        }, file, BatchTaskTypeEnum.DEFAULT,ExcelUtil.BatchImportStrategy.ROLL_BACK);
    }

    @Override
    public XSSFWorkbook exportExcel(UserPost userPost) {
        XSSFWorkbook wb = ExcelUtil.export(UserPost.class,
        (page, userPostReq) -> selectPage(page,userPostReq),
        userPost);
        return wb;
    }

    @Override
    public List<SimplePostModel> getPostList(Long userId) {
        List<SimplePostModel> list = userPostMapper.getPostList(userId);
        return list;
    }

    @Override
    public void saveOrUpdateById(UserPost userPost) {
        userPostMapper.saveOrUpdateById(userPost);
    }

}
